def encode_file_path(path: str | bytes) -> bytes: ...
def tobytes(o: str | bytes) -> bytes: ...
def frombytes(o: bytes, *, safe: bool = False): ...

__all__ = ["encode_file_path", "tobytes", "frombytes"]
